/******************************************************************************
* FILENAME:
* ---------
* mac_phy_test.c
*
* DESCRIPTION:
* ------------
* This program tests the basic functionality of the
* EMAC controller and PHY on the Xilinx PMP board. Tests carried out
* are:
* 1. EMAC self test
* 2. EMAC internal loop back test (Without auto PAD and FCS stripping)
* 3. EMAC internal loop back test (With auto PAD and FCS stripping)
* 4. EMAC address filtering test
* 5. PHY register read and write test
* 6. PHY loop back test
*
* AUTHOR:
* -------
* Welson Sun
*
******************************************************************************/
/***************************** Include Files *********************************/
#include <xparameters.h>
#include <xstatus.h>
#include "peripheral.h"
#include "mac_phy_test.h"
#include "opb_onewire_l.h"

/*
 * The main() method for all the tests.
 * TO-DO:
 *       Solve the auto-pad-fcs-strip issue
 */
int main()
{
    XEmac Emac;
    XEmac *InstancePtr = &Emac;
    Xuint16 DeviceId = XPAR_ETHERNET_MAC_DEVICE_ID;
    
    Xuint8 mac1;



	init_uart();

	initLedDip();

    
    xil_printf("MAC PHY TEST\r\n");
    
    //0. Initialize the EMAC
    if( emac_init(InstancePtr, DeviceId) )
        return;

    //1. Self test
    if( emac_selfTest(InstancePtr) )
        return;

    //2. EMAC loop back test(No PAD strip on receiving)
    xil_printf("*** Test 2: EMAC loop back testing (no PAD strip on receiving)\r\n");
    emac_loopBackTest(InstancePtr);

    //3. EMAC loop back test(Auto PAD strip on receiving)
    if( emac_init(InstancePtr, DeviceId) )
        return;
    xil_printf("*** Test 3: EMAC loop back testing (with PAD strip on receiving)\r\n");
    emac_stripLoopBackTest(InstancePtr);

    //4. EMAC address filter test
    if( emac_init(InstancePtr, DeviceId) )
        return;
    emac_addressTest(InstancePtr);

    //5. EMAC PHY communication test
    emac_phyTest(InstancePtr, 0);

    //6. PHY loop back test, four combinations of FullDuplex/FastEthernet
    {
        Xboolean success = XTRUE;
        
        xil_printf("*** Test 6: PHY loop back testing......\r\n");
        FastEthernet = XTRUE;
        
        FullDuplex = XTRUE;
        xil_printf("***100BaseT, Full Duplex***\r\n");
        if(emac_phyLoopBackTest(InstancePtr))
            success = XFALSE;

        FullDuplex = XFALSE;
        xil_printf("***100BaseT, Half Duplex***\r\n");
        if(emac_phyLoopBackTest(InstancePtr))
            success = XFALSE;                                           
        
        FastEthernet = XFALSE;

        FullDuplex = XTRUE;
        xil_printf("***10BaseT, Full Duplex***\r\n");
        if(emac_phyLoopBackTest(InstancePtr))
            success = XFALSE;
        
        FullDuplex = XFALSE;
        xil_printf("***10BaseT, Half Duplex***\r\n");
        if(emac_phyLoopBackTest(InstancePtr))
            success = XFALSE;

        if(success){
            xil_printf("All PHY loop back test SUCCESSFUL!\r\n");
        }
        else
            xil_printf("PHY loop back test FAILED!\r\n");
    }
    
    return 0;
}

//Initialize the EMAC controller
int emac_init(XEmac *InstancePtr, Xuint16 DeviceId){
    
    XStatus Result;
	unsigned char hw_addr[ETH_ADDR_LEN];
    
    Result = XEmac_Initialize(InstancePtr, DeviceId);
    
    //Stop and re-init if EMAC has already been started
    if (Result == XST_DEVICE_IS_STARTED){
        (void)XEmac_Stop(InstancePtr);
        Result = XEmac_Initialize(InstancePtr, DeviceId);
    }
    
    //Check the result
    if (Result != XST_SUCCESS){
        xil_printf("EMAC initialized FAILED!\r\n");
        return 1;
    }
    
	OneWire_GetHardwareAddr(XPAR_ONEWIRE_0_BASEADDR,LocalAddress);
//	LocalAddress[0]=hw_addr[0];
//	LocalAddress[1]=hw_addr[2];
//	LocalAddress[2]=hw_addr[4];
// 	LocalAddress[3]=hw_addr[6];
//	LocalAddress[4]=hw_addr[8];
//	LocalAddress[5]=hw_addr[10];
  	xil_printf("ETH_HW_ADDR: %02x:%02x:%02x:%02x:%02x:%02x \r\n",\
		LocalAddress[0],LocalAddress[1],LocalAddress[2],
		LocalAddress[3],LocalAddress[4],LocalAddress[5]);
    xil_printf("EMAC initialized SUCCESSFUL!\r\n");


    return 0;
}

//Set the EMAC option
int emac_setMACOptions(XEmac *InstancePtr, Xuint32 Options)
{
    Xuint16 RegisterNum;
    Xuint16 PhyData;
    XStatus Result;
    
    //Stop EMAC 
    XEmac_Stop(InstancePtr);


    //Set the EMAC option, turn on full duplex if needed
    if(FullDuplex)
        Result = XEmac_SetOptions(InstancePtr, Options | XEM_FDUPLEX_OPTION);
    else
        Result = XEmac_SetOptions(InstancePtr, Options);

    //Check the result
    if (Result != XST_SUCCESS){
        xil_printf("EMAC option setting FAILED!\r\n");
        return 1;
    }
    
    return 0;
}

//EMAC self test
int emac_selfTest(XEmac *InstancePtr)
{
    //Set the EMAC option for self test
    Xuint32 Options;
    XStatus Result;

    Options = (XEM_UNICAST_OPTION |
               XEM_BROADCAST_OPTION |
               XEM_POLLED_OPTION |
               XEM_INSERT_PAD_OPTION |
               XEM_INSERT_FCS_OPTION |
               XEM_INSERT_ADDR_OPTION |
               XEM_OVWRT_ADDR_OPTION);
    
    if( emac_setMACOptions(InstancePtr, Options) )
        return 1;

    //Run self test
    xil_printf("*** Test 1: EMAC self testing......\r\n");
    Result = XEmac_SelfTest(InstancePtr);

    //Check result
    if (Result != XST_SUCCESS){
        xil_printf("EMAC self test FAILED!\r\n");
        return 1;
    }

    xil_printf("EMAC self test SUCCESSFUL!\r\n");
    return 0;
}

// EMAC loop back test, with no auto PAD and FCS strip function
int emac_loopBackTest(XEmac *InstancePtr)
{
    XStatus Result;
    Xuint32 Options;
    
    //Stop the MAC
    XEmac_Stop(InstancePtr);

    //Set the local MAC address
    Result = XEmac_SetMacAddress(InstancePtr, LocalAddress);
    if (Result != XST_SUCCESS){
        xil_printf("MAC address setting FAILED!\r\n");
        return 1;
    }
    
    //Set the loopback option
    Options = (XEM_POLLED_OPTION |
               XEM_PROMISC_OPTION |
               XEM_INSERT_PAD_OPTION |
               XEM_INSERT_FCS_OPTION |
               XEM_INSERT_ADDR_OPTION |
               XEM_OVWRT_ADDR_OPTION |
               XEM_LOOPBACK_OPTION ); //Enable the loopback mode

    if(emac_setMACOptions(InstancePtr, Options))
        return 1;

    //Start the MAC
    Result = XEmac_Start(InstancePtr);
    if (Result != XST_SUCCESS){
        xil_printf("MAC start FAILED!\r\n");
        return 1;
    }

    //Loop back test from 46 bytes to 1500 bytes
    {
        int Size;
        Xboolean success = XTRUE;
        for (Size=46; Size <= XEM_MTU; Size ++)
            if(emac_loopBackFrame(InstancePtr, Size)){
                xil_printf("MAC loop back test FAILED with frame size: %d\r\n", Size);
                success = XFALSE;
                break;
            }
        if(success)
            xil_printf("MAC loop back test 1 SUCCESSFUL!\r\n");
        else
            return 1;
    }

    //Send two frames successively, and then receive them
    {
        Xboolean success = XFALSE;

        if( emac_sendFrame(InstancePtr, 60, LocalAddress) == 0)
            if ( emac_sendFrame(InstancePtr, 120, LocalAddress) == 0)
                if ( emac_receiveFrame(InstancePtr, 60, XTRUE) == 0)
                    if (emac_receiveFrame(InstancePtr, 120, XTRUE) == 0)
                        success = XTRUE;
        if(success)
            xil_printf("MAC loop back test 2 SUCCESSFUL!\r\n");
        else{
            xil_printf("MAC loop back test 2 FAILED!\r\n");
            return 1;
        }
    }
    
    //Send 16 frames successively (the length/status FIFO depths) and receive them
    {
        int i;
        Xboolean send_success = XTRUE;
        Xboolean rec_success = XTRUE;
        
        for (i=60; i < 76; i++)
            if(emac_sendFrame(InstancePtr, i, LocalAddress)){
                send_success = XFALSE;
                break;
            }
        if(!send_success){
            xil_printf("MAC loop back test 3 FAILED!\r\n");
            return 1;
        }
        else{
            for (i=60; i < 76; i++)
                if(emac_receiveFrame(InstancePtr, i, XTRUE)){
                    rec_success = XFALSE;
                    break;
                }
            if(!rec_success){
                xil_printf("MAC loop back test 3 FAILED!\r\n");
                return 1;
            }
            else
                xil_printf("MAC loop back test 3 SUCCESSFUL!\r\n");
        }
    }
    
    //Send three frames successively(1500,1500, 1000) and receive them.
    {
        Xboolean success = XFALSE;
        
        if( emac_sendFrame(InstancePtr, 60, LocalAddress) == 0)
            if ( emac_sendFrame(InstancePtr, 120, LocalAddress) == 0)
                if ( emac_sendFrame(InstancePtr, 120, LocalAddress) == 0)
                    if ( emac_receiveFrame(InstancePtr, 60, XTRUE) == 0)
                        if (emac_receiveFrame(InstancePtr, 120, XTRUE) == 0)
                            if (emac_receiveFrame(InstancePtr, 120, XTRUE) == 0)
                                success = XTRUE;
        if(success)
            xil_printf("MAC loop back test 4 SUCCESSFUL!\r\n");
        else{
            xil_printf("MAC loop back test 4 FAILED!\r\n");
            return 1;
        }
    }

}

//Helper method: send an arbitrary length(46-1500) frame and check the
//received frame with the send frame
int emac_loopBackFrame(XEmac *InstancePtr, Xuint32 PayloadSize)
{
    if( emac_sendFrame(InstancePtr, PayloadSize, LocalAddress) == 0 ){
        if (emac_receiveFrame(InstancePtr, PayloadSize, XTRUE) == 0){
            return 0;
        }
    }
    xil_printf("Loop back frame test FAIL!\r\n");
    return 1;
}

//Helper method: send an arbitrary length frame
int emac_sendFrame(XEmac *InstancePtr, Xuint32 PayloadSize, Xuint8 *DestAddress)
{
    Xuint32 Frame[XEM_MAX_FRAME_SIZE_IN_WORDS];
    Xuint8 *FramePtr;
    Xuint8 *AddrPtr = DestAddress;
    int Index;
    XStatus Result;

    FramePtr = (Xuint8 *)Frame;

    //Set the destination MAC address
    *FramePtr++ = *AddrPtr++;
    *FramePtr++ = *AddrPtr++;
    *FramePtr++ = *AddrPtr++;
    *FramePtr++ = *AddrPtr++;
    *FramePtr++ = *AddrPtr++;
    *FramePtr++ = *AddrPtr++;

    //Skip the source MAC address, the caller should be responsible
    //for setting the INSERT_SOURCE_ADDRESS and
    //OVERWRITE_SOURCE_ADDRESS option
    FramePtr += XEM_MAC_ADDR_SIZE;

    //Set up the type/length field - be sure its in network order
    *((Xuint16 *)FramePtr) = PayloadSize;
    FramePtr++;
    FramePtr++;

    //Now fill the payload with known pattern
    for (Index = 0; Index < PayloadSize; Index++)
        *FramePtr++ = (Xuint8)Index;
    
    //Send the frame
    Result = XEmac_PollSend(InstancePtr, (Xuint8 *)Frame, PayloadSize + XEM_HDR_SIZE);
    if (Result != XST_SUCCESS){
        xil_printf("EMAC send frame FAILED!\r\n");
        return Result;
    }
    else
        return 0;
}

//Helper method: receive an arbitrary length frame
int emac_receiveFrame(XEmac *InstancePtr, Xuint32 PayloadSize, Xboolean Verify)
{
    //This is assuming that the payload size is larger than 46 bytes.
    Xuint32 Frame[XEM_MAX_FRAME_SIZE_IN_WORDS];
    Xuint8 *FramePtr;
    XStatus Result;
    Xuint32 RecvFrameLength;

    RecvFrameLength = XEM_MAX_FRAME_SIZE; //You have to do this before receiving
    Result = XEmac_PollRecv(InstancePtr, (Xuint8 *)Frame, &RecvFrameLength);
    if (Result == XST_SUCCESS){
        int Index;

        /* Verify length, which should be the payload plus overhead */
        if (RecvFrameLength != (PayloadSize + XEM_HDR_SIZE + XEM_TRL_SIZE))
        {
            xil_printf("Receive frame length ERROR!\r\n");
            return 1;
        }

        /* Verify the frame contents */
        if (Verify)
        {
            FramePtr = (Xuint8 *)Frame;
            FramePtr += XEM_HDR_SIZE;
            for (Index = 0; Index < PayloadSize; Index++){
                if (*FramePtr++ != (Xuint8)Index){
                    xil_printf("Receive frame data ERROR!\r\n");
                    return 2;
                }
            }
        }
        return 0;
    }
    else
        return Result;
}

//EMAC address filter test
int emac_addressTest(XEmac *InstancePtr){
    Xuint32 Options;
    XStatus Result;
    int testNo = 0;

    Xboolean success = XTRUE;
    
    xil_printf("*** Test 3: EMAC address filtering testing......\r\n");

    //Stop the MAC
    XEmac_Stop(InstancePtr);

    //Set the local MAC address
    Result = XEmac_SetMacAddress(InstancePtr, LocalAddress);
    if (Result != XST_SUCCESS){
        xil_printf("MAC address setting FAILED!\r\n");
        return 1;
    }

    /********************** Testing UNICAST mode ******************/
    Options = (XEM_LOOPBACK_OPTION |
               XEM_POLLED_OPTION |
               XEM_UNICAST_OPTION |
               XEM_INSERT_PAD_OPTION |
               XEM_INSERT_FCS_OPTION);
    if (emac_setMACOptions(InstancePtr, Options))
        success = XFALSE;

    Result = XEmac_Start(InstancePtr);
    if (Result != XST_SUCCESS){
        xil_printf("MAC start FAILED!\r\n");
        return 1;
    }
    
    //Send to local address
    if(addrTest(++testNo, LocalAddress, InstancePtr, XTRUE))
        success = XFALSE;

    //Send to a BROADCAST address
    if(addrTest(++testNo, BroadcastAddress, InstancePtr, XFALSE))
        success = XFALSE;

    //Send to a remote address
    if(addrTest(++testNo, RemoteAddress, InstancePtr, XFALSE))
        success = XFALSE;

    /********************** Testing BROADCAST mode ******************/
    Options = (XEM_LOOPBACK_OPTION |
               XEM_POLLED_OPTION |
               XEM_BROADCAST_OPTION |
               XEM_INSERT_PAD_OPTION |
               XEM_INSERT_FCS_OPTION);

    if(emac_setMACOptions(InstancePtr, Options))
        return 1;

    Result = XEmac_Start(InstancePtr);
    if (Result != XST_SUCCESS){
        xil_printf("MAC start FAILED!\r\n");
        return 1;
    }

    //Send to a broadcast address
    if(addrTest(++testNo, BroadcastAddress, InstancePtr, XTRUE))
        success = XFALSE;

    //Send to a local address
    if(addrTest(++testNo, LocalAddress, InstancePtr, XFALSE))
        success = XFALSE;

    //Send to a remote address
    if(addrTest(++testNo, RemoteAddress, InstancePtr, XFALSE))
        success = XFALSE;


    /********************** Testing BROADCAST and UNICAST mode ******************/
    Options = (XEM_LOOPBACK_OPTION |
               XEM_POLLED_OPTION |
               XEM_UNICAST_OPTION |
               XEM_BROADCAST_OPTION |
               XEM_INSERT_PAD_OPTION |
               XEM_INSERT_FCS_OPTION );
    
    if(emac_setMACOptions(InstancePtr, Options))
        return 1;
    
    Result = XEmac_Start(InstancePtr);
    if (Result != XST_SUCCESS){
        xil_printf("MAC start FAILED!\r\n");
        return 1;
    }

    //Send to local address
    if(addrTest(++testNo, LocalAddress, InstancePtr, XTRUE))
        success = XFALSE;

    //Send to a remote address
    if(addrTest(++testNo, RemoteAddress, InstancePtr, XFALSE))
        success = XFALSE;
    
    //Send to a broadcast address
    if(addrTest(++testNo, BroadcastAddress, InstancePtr, XTRUE))
        success = XFALSE;

    /********************** Testing Promiscuous Mode ******************/
    Options = (XEM_LOOPBACK_OPTION |
               XEM_POLLED_OPTION |
               XEM_PROMISC_OPTION |
               XEM_INSERT_PAD_OPTION |
               XEM_INSERT_FCS_OPTION);
    
    if(emac_setMACOptions(InstancePtr, Options))
        return 1;
    
    Result = XEmac_Start(InstancePtr);
    if (Result != XST_SUCCESS){
        xil_printf("MAC start FAILED!\r\n");
        return 1;
    }

    //Send to local address
    if(addrTest(++testNo, LocalAddress, InstancePtr, XTRUE))
        success = XFALSE;

    //Send to remote address
    if(addrTest(++testNo, RemoteAddress, InstancePtr, XTRUE))
        success = XFALSE;                                                           

    //Send to a broadcast address
    if(addrTest(++testNo, BroadcastAddress, InstancePtr, XTRUE))
        success = XFALSE;


    /********************** Testing Promiscuous and UNICAST Mode ******************/
    Options = (XEM_LOOPBACK_OPTION |
               XEM_POLLED_OPTION |
               XEM_UNICAST_OPTION |
               XEM_PROMISC_OPTION |
               XEM_INSERT_PAD_OPTION |
               XEM_INSERT_FCS_OPTION);

    if(emac_setMACOptions(InstancePtr, Options))
        return 1;
    
    Result = XEmac_Start(InstancePtr);
    if (Result != XST_SUCCESS){
        xil_printf("MAC start FAILED!\r\n");
        return 1;
    }

    //Send to local address
    if(addrTest(++testNo, LocalAddress, InstancePtr, XTRUE))
        success = XFALSE;

    //Send to remote address
    if(addrTest(++testNo, RemoteAddress, InstancePtr, XTRUE))
        success = XFALSE;

    //Send to a broadcast address
    if(addrTest(++testNo, BroadcastAddress, InstancePtr, XTRUE))
        success = XFALSE;

    /********************** Testing Promiscuous and BROADCAST Mode ******************/
    Options = (XEM_LOOPBACK_OPTION |
               XEM_POLLED_OPTION |
               XEM_BROADCAST_OPTION |
               XEM_PROMISC_OPTION |
               XEM_INSERT_PAD_OPTION |
               XEM_INSERT_FCS_OPTION);

    if(emac_setMACOptions(InstancePtr, Options))
        return 1;
    
    Result = XEmac_Start(InstancePtr);
    if (Result != XST_SUCCESS){
        xil_printf("MAC start FAILED!\r\n");
        return 1;
    }

    //Send to local address
    if(addrTest(++testNo, LocalAddress, InstancePtr, XTRUE))
        success = XFALSE;
    
    //Send to remote address
    if(addrTest(++testNo, RemoteAddress, InstancePtr, XTRUE))
        success = XFALSE;
    
    //Send to a broadcast address
    if(addrTest(++testNo, BroadcastAddress, InstancePtr, XTRUE))
        success = XFALSE;


    /********************** Testing MULTICAST Mode ******************/
    Options = (XEM_LOOPBACK_OPTION |
               XEM_POLLED_OPTION |
               XEM_MULTICAST_OPTION |
               XEM_INSERT_PAD_OPTION |
               XEM_INSERT_FCS_OPTION);
    if(emac_setMACOptions(InstancePtr, Options))
        return 1;
    
    Result = XEmac_Start(InstancePtr);
    if (Result != XST_SUCCESS){
        xil_printf("MAC start FAILED!\r\n");
        return 1;
    }

    //Send to Multicast address
    if(addrTest(++testNo, MulticastAddress, InstancePtr, XTRUE))
        success = XFALSE;
    
    //Send to remote address
    if(addrTest(++testNo, RemoteAddress, InstancePtr, XFALSE))
        success = XFALSE;

    //Send to a broadcast address
    if(addrTest(++testNo, BroadcastAddress, InstancePtr, XFALSE))
        success = XFALSE;
    
    if(success){
        return 0;
    }
    else
        return 1;
   
}
    
//Helper method, send a frame out and check if it is received or not,
//return the status according to the parameter: shouldReceive
int addrTest(int index, Xuint8 *DA, XEmac *InstancePtr, Xboolean shouldReceive){
    Xboolean success;

    if(shouldReceive){
        success = XFALSE;
        if (emac_sendFrame(InstancePtr, 256, DA) == 0)
            if (emac_receiveFrame(InstancePtr, 256, XTRUE) == 0)
                success = XTRUE;
        if (success){
            xil_printf("EMAC address filter test %d SUCCESSFUL!\r\n", index);
            return 0;
        }
        else{
            xil_printf("EMAC address filter test %d FAILED!\r\n", index);
            return 1;
        }
    }
    else{
        int i, j;
        success = XTRUE;
        emac_sendFrame(InstancePtr, 256, DA);
        for (i=0; i < 5; i++){
            if ( emac_receiveFrame(InstancePtr, 256, XTRUE) != XST_NO_DATA ){
                success = XFALSE;
                break;
            }
            for (j=0; j<100; j++);
        }
        if (success){
            xil_printf("EMAC address filter test %d SUCCESSFUL!\r\n", index);
            return 0;
        }
        else{
            xil_printf("EMAC address filter test %d FAILED!\r\n", index);
            return 1;
        }
    }
}

//EMAC internal loop back test, with the auto PAD and FCS strip on
int emac_stripLoopBackTest(XEmac *InstancePtr)
{
    Xuint32 Options;
    XStatus Result;
    
    //Stop the MAC
    XEmac_Stop(InstancePtr);

    //Set the local MAC address
    Result = XEmac_SetMacAddress(InstancePtr, LocalAddress);
    if (Result != XST_SUCCESS){
        xil_printf("MAC address setting FAILED!\r\n");
        return 1;
    }

    Options = (XEM_POLLED_OPTION |
               XEM_LOOPBACK_OPTION |
               XEM_PROMISC_OPTION |
               XEM_INSERT_PAD_OPTION |
               XEM_INSERT_FCS_OPTION |
               XEM_INSERT_ADDR_OPTION |
               XEM_OVWRT_ADDR_OPTION |
               XEM_STRIP_PAD_FCS_OPTION);

    if(emac_setMACOptions(InstancePtr, Options))
        return 1;
    
    Result = XEmac_Start(InstancePtr);
    if (Result != XST_SUCCESS){
        xil_printf("MAC start FAILED!\r\n");
        return 1;
    }

    {
        Xboolean success = XTRUE;
        
        int Size;
        for (Size=45; Size<48; Size++){
            if (emac_sendFrame(InstancePtr, Size, LocalAddress) ){
                success = XFALSE;
                break;
            }
            xil_printf("Sent frame size: %d\r\n", Size);
            
            if (emac_receivePFrame(InstancePtr,(Size+XEM_HDR_SIZE),XFALSE) ){
                success = XFALSE;
                //break;
            }
        }
        if(success){
            xil_printf("EMAC strip loop back test SUCCESSFUL!\r\n");
            return 0;
        }
        else{
            xil_printf("EMAC strip loop back test FAILED!\r\n");
            return 1;
        }
    }
}

//Helper method: Receive a frame with PAD and FCS stripped off
int emac_receivePFrame(XEmac *InstancePtr, Xuint32 PayloadSize, Xboolean Verify)
{
    //This is assuming that the payload size is larger than 46 bytes.
    Xuint32 Frame[XEM_MAX_FRAME_SIZE_IN_WORDS];
    Xuint8 *FramePtr;
    XStatus Result;
    Xuint32 RecvFrameLength;

    RecvFrameLength = XEM_MAX_FRAME_SIZE; //You have to do this before receiving
    Result = XEmac_PollRecv(InstancePtr, (Xuint8 *)Frame, &RecvFrameLength);
    if (Result == XST_SUCCESS){
        int Index;
        int i;

        /* DUMPING THE RECEIVED FRAME */
        FramePtr = (Xuint8 *)Frame;
        for(i=0; i<RecvFrameLength; i++)
            xil_printf("%X ", *FramePtr++);
        xil_printf("\r\n");
        
        /* Verify length, which should be the payload plus overhead */
        if (RecvFrameLength != (PayloadSize))
        {
            xil_printf("Receive frame length: %d\r\n", RecvFrameLength);
            return 1;
        }

        /* Verify the frame contents */
        if (Verify)
        {
            FramePtr = (Xuint8 *)Frame;
            FramePtr += XEM_HDR_SIZE;    /* get past the header */
            for (Index = 0; Index < PayloadSize-XEM_HDR_SIZE; Index++){
                if (*FramePtr++ != (Xuint8)Index){
                    xil_printf("Receive frame data ERROR!\r\n");
                    return 2;
                }
            }
        }

        return 0;
    }
    else{
        xil_printf("Receive error! %d", Result);
        return Result;
    }
}

//PHY register read and write test
int emac_phyTest(XEmac *InstancePtr, Xuint8 PhyAddress)
{
    Xuint16 PHY_ID1 = 0x0013;
    Xuint16 PHY_ID2 = 0x78E2;

    Xuint16 PhyData;
    const char *PhyName = "LXT972A";
    Xboolean success = XTRUE;
    
    xil_printf("*** Test 5: %s testing......\r\n", PhyName);

    //Read control register
    if (XEmac_PhyRead(InstancePtr, PhyAddress, 0, &PhyData)){
        xil_printf("PHY control register read FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY control register read SUCCESSFUL, Value = 0x%X\r\n", PhyData);
        
    
    //Reset PHY, read control register and status register
    if( XEmac_PhyWrite(InstancePtr, PhyAddress, 0, 0x8000) ){
        xil_printf("PHY reset FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY reset SUCCESSFUL!\r\n");

    if ( XEmac_PhyRead(InstancePtr, PhyAddress, 0, &PhyData) ){
        xil_printf("PHY control register read FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY control register read SUCCESSFUL, Value = 0x%X\r\n", PhyData);

    if( XEmac_PhyRead(InstancePtr, PhyAddress, 1, &PhyData) ){
        xil_printf("PHY status register #1 read FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY status register #1 read SUCCESSFUL, Value = 0x%X\r\n", PhyData);
        

    //Check PHY identification register
    if( XEmac_PhyRead(InstancePtr, PhyAddress, 2, &PhyData) ){
        xil_printf("PHY identification register 1 read FAILED!\r\n");
        success = XFALSE;
    }
    else
        if (PhyData != PHY_ID1)
            xil_printf("%s: ID register 1: 0x%X does not match 0x%x.\r\n", PhyName, PhyData, PHY_ID1);
        else
            xil_printf("PHY identification register 1 check SUCCESSFUL!\r\n");
    
    if( XEmac_PhyRead(InstancePtr, PhyAddress, 3, &PhyData) ){
        xil_printf("PHY identification register 2 read FAILED!\r\n");
        success = XFALSE;
    }
    else
        if (PhyData != PHY_ID2)
            xil_printf("%s: ID register 2: 0x%X does not match 0x%x.\r\n", PhyName, PhyData, PHY_ID2);
        else
            xil_printf("PHY identification register 2 check SUCCESSFUL!\r\n");
    
    // Auto Negotiation advertisement register (ANAR)
    if( XEmac_PhyRead(InstancePtr, PhyAddress, 4, &PhyData) ){
        xil_printf("PHY register 4 read FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY register 4 read SUCCESSFUL, Value = 0x%X\r\n", PhyData);
    
    // Auto Negotiation link partner ability (ANLPAR)
    if( XEmac_PhyRead(InstancePtr, PhyAddress, 5, &PhyData) ){
        xil_printf("PHY register 5 read FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY register 5 read SUCCESSFUL, Value = 0x%X\r\n", PhyData);
    
    // Auto Negotiation expansion
    if( XEmac_PhyRead(InstancePtr, PhyAddress, 6, &PhyData) ){
        xil_printf("PHY register 6 read FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY register 6 read SUCCESSFUL, Value = 0x%X\r\n", PhyData);

    // Mirror register
    if( XEmac_PhyRead(InstancePtr, PhyAddress, 16, &PhyData) ){
        xil_printf("PHY register 16 read FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY register 16 read SUCCESSFUL, Value = 0x%X\r\n", PhyData);

    // Interrupt enable register
    if( XEmac_PhyRead(InstancePtr, PhyAddress, 17, &PhyData) ){
        xil_printf("PHY register 17 read FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY register 17 read SUCCESSFUL, Value = 0x%X\r\n", PhyData);

    // Interrupt status register
    if( XEmac_PhyRead(InstancePtr, PhyAddress, 18, &PhyData) ){
        xil_printf("PHY register 18 read FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY register 18 read SUCCESSFUL, Value = 0x%X\r\n", PhyData);

    // Configuration register
    if( XEmac_PhyRead(InstancePtr, PhyAddress, 19, &PhyData) ){
        xil_printf("PHY register 19 read FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY register 19 read SUCCESSFUL, Value = 0x%X\r\n", PhyData);

    // Chip status register
    if( XEmac_PhyRead(InstancePtr, PhyAddress, 20, &PhyData) ){
        xil_printf("PHY register 20 read FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY register 20 read SUCCESSFUL, Value = 0x%X\r\n", PhyData);


    // Enable interrupt register
    if( XEmac_PhyWrite(InstancePtr, PhyAddress, 17, 0x000C)){
        xil_printf("PHY interrupt enable FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY interrupt enable SUCCESSFUL!\r\n");

    if( XEmac_PhyRead(InstancePtr, PhyAddress, 17, &PhyData) ){
        xil_printf("PHY interrupt register read FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY interrupt register read SUCCESSFUL, Value = 0x%X\r\n", PhyData);
    

    // Enable loopback, read back, disable loopback
    if( XEmac_PhyWrite(InstancePtr, PhyAddress, 19, 0x8000)){
        xil_printf("PHY loopback enable FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY loopback enable SUCCESSFUL!\r\n");

    if( XEmac_PhyRead(InstancePtr, PhyAddress, 19, &PhyData) ){
        xil_printf("PHY register 19 read FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY register 19 read SUCCESSFUL, Value = 0x%X\r\n", PhyData);

    if( XEmac_PhyWrite(InstancePtr, PhyAddress, 19, 0x0000)){
        xil_printf("PHY loopback disable FAILED!\r\n");
        success = XFALSE;
    }
    else
        xil_printf("PHY loopback disable SUCCESSFUL!\r\n");

    if(success){
        return 0;
    }
    else
        return 1;
}

//PHY loop back test
int emac_phyLoopBackTest(XEmac *InstancePtr)
{
    Xuint32 Options;

    
    emac_setPHYOptions(InstancePtr, XTRUE);
    //Force link up
    XEmac_PhyWrite(InstancePtr, 0, 16, 0x4000);

    XEmac_Stop(InstancePtr);

    //Set the local MAC address
    if( XEmac_SetMacAddress(InstancePtr, LocalAddress) ){
        xil_printf("MAC address setting FAILED!\r\n");
        return 1;
    }

    Options = (XEM_POLLED_OPTION |
               XEM_UNICAST_OPTION |
               XEM_INSERT_PAD_OPTION |
               XEM_INSERT_FCS_OPTION |
               XEM_INSERT_ADDR_OPTION |
               XEM_OVWRT_ADDR_OPTION);

    if (emac_setMACOptions(InstancePtr, Options))
        return 1;

    if( XEmac_Start(InstancePtr) ){
        xil_printf("MAC start FAILED!\r\n");
        return 1;
    }

    //Loop back test from 46 bytes to 1500 bytes
    {
        int Size;
        Xboolean success = XTRUE;
        for (Size=46; Size <= XEM_MTU; Size ++)
            if(emac_loopBackFrame(InstancePtr, Size)){
                xil_printf("PHY loop back test FAILED with frame size: %d\r\n", Size);
                success = XFALSE;
                break;
            }
        
        if(success)
            xil_printf("PHY loop back test 1 SUCCESSFUL!\r\n");
        else
            return 1;
    }

    //Send two frames successively, and then receive them
    {
        Xboolean success = XFALSE;

        if( emac_sendFrame(InstancePtr, 60, LocalAddress) == 0)
            if ( emac_sendFrame(InstancePtr, 120, LocalAddress) == 0)
                if ( emac_receiveFrame(InstancePtr, 60, XTRUE) == 0)
                    if (emac_receiveFrame(InstancePtr, 120, XTRUE) == 0)
                        success = XTRUE;
        if(success)
            xil_printf("PHY loop back test 2 SUCCESSFUL!\r\n");
        else{
            xil_printf("PHY loop back test 2 FAILED!\r\n");
            return 1;
        }
    }
    
    //Send 16 frames successively (the length/status FIFO depths) and receive them
    {
        int i;
        Xboolean send_success = XTRUE;
        Xboolean rec_success = XTRUE;
        
        for (i=60; i < 76; i++)
            if(emac_sendFrame(InstancePtr, i, LocalAddress)){
                send_success = XFALSE;
                break;
            }
        if(!send_success){
            xil_printf("PHY loop back test 3 FAILED!\r\n");
            return 1;
        }
        else{
            for (i=60; i < 76; i++)
                if(emac_receiveFrame(InstancePtr, i, XTRUE)){
                    rec_success = XFALSE;
                    break;
                }
            if(!rec_success){
                xil_printf("PHY loop back test 3 FAILED!\r\n");
                return 1;
            }
            else
                xil_printf("PHY loop back test 3 SUCCESSFUL!\r\n");
        }
    }
    
    //Send three frames successively(1500,1500, 1000) and receive them.
    {
        Xboolean success = XFALSE;
        
        if( emac_sendFrame(InstancePtr, 60, LocalAddress) == 0)
            if ( emac_sendFrame(InstancePtr, 120, LocalAddress) == 0)
                if ( emac_sendFrame(InstancePtr, 120, LocalAddress) == 0)
                    if ( emac_receiveFrame(InstancePtr, 60, XTRUE) == 0)
                        if (emac_receiveFrame(InstancePtr, 120, XTRUE) == 0)
                            if (emac_receiveFrame(InstancePtr, 120, XTRUE) == 0)
                                success = XTRUE;
        if(success)
            xil_printf("PHY loop back test 4 SUCCESSFUL!\r\n");
        else{
            xil_printf("PHY loop back test 4 FAILED!\r\n");
            return 1;
        }
    }

    return 0;
}

//Set the PHY option
int emac_setPHYOptions(XEmac *InstancePtr, Xboolean loopBack)
{
    Xuint16 PhyData;
    PhyData = 0;

    //Reset PHY, read control register and status register
    if( XEmac_PhyWrite(InstancePtr, 0, 0, 0x8000) )
        xil_printf("PHY reset FAILED!\r\n");
    else
        xil_printf("PHY reset SUCCESSFUL!\r\n");
    
    //Set the PHY into full duplex if needed
    if (XEmac_PhyRead(InstancePtr, 0, 0, &PhyData))
        return 1;

    PhyData &= 0xEFFF; //Disable auto negotiation
    
    if(FullDuplex)
        PhyData |= 0x0100;
    else
        PhyData &= 0xFEFF;

    if(FastEthernet)
        PhyData |= 0x2000;
    else
        PhyData &= 0xDFFF;
    
    if(loopBack)
        PhyData |= 0x4000;
    else
        PhyData &= 0xBFFF;

    if (XEmac_PhyWrite(InstancePtr, 0, 0, PhyData))
        return 1;

    
    if (XEmac_PhyRead(InstancePtr, 0, 17, &PhyData))
        return 1;
    
    return 0;
}
